<?php

namespace Drupal\Tests\encrypt_rsa\Kernel;

use Drupal\encrypt\Exception\EncryptionMethodCanNotDecryptException;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests RSA encryption method.
 *
 * @group encrypt_rsa
 */
class RSATest extends KernelTestBase {

  /**
   * Plain text message string.
   */
  const PLAIN_TEXT_MESSAGE = 'Hello World!';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'key',
    'key_asymmetric',
    'encrypt',
    'encrypt_rsa',
    'encrypt_rsa_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig('encrypt_rsa_test');
  }

  /**
   * Test public profile/method/key.
   */
  public function testPublicEncryptDecrypt() {

    /** @var \Drupal\encrypt\EncryptionProfileInterface $profile */
    $profile = $this->container->get('entity_type.manager')
      ->getStorage('encryption_profile')
      ->load('rsa_public');

    $text = $profile->getEncryptionMethod()->encrypt(static::PLAIN_TEXT_MESSAGE, $profile->getEncryptionKey()->getKeyValue());

    // Check text is encrypted.
    $this->assertNotEquals(static::PLAIN_TEXT_MESSAGE, $text);

    // Check decrypting throws an exception.
    $this->expectException(EncryptionMethodCanNotDecryptException::class);
    $this->expectExceptionMessage('Decrypting with a public key is not possible.');
    $profile->getEncryptionMethod()->decrypt($text, $profile->getEncryptionKey()->getKeyValue());

    // Manually encrypt and confirm all is good.
    $profile = $this->container->get('entity_type.manager')
      ->getStorage('encryption_profile')
      ->load('rsa_private');

    $this->assertEquals(static::PLAIN_TEXT_MESSAGE, $profile->getEncryptionMethod()->decrypt($text, $profile->getEncryptionKey()->getKeyValue()));

  }

  /**
   * Test private profile/method/key.
   */
  public function testPrivateEncryptDecrypt() {

    /** @var \Drupal\encrypt\EncryptionProfileInterface $profile */
    $profile = $this->container->get('entity_type.manager')
      ->getStorage('encryption_profile')
      ->load('rsa_private');

    $text = $profile->getEncryptionMethod()->encrypt(static::PLAIN_TEXT_MESSAGE, $profile->getEncryptionKey()->getKeyValue());

    // Check text is encrypted.
    $this->assertNotEquals(static::PLAIN_TEXT_MESSAGE, $text);

    // Check decrypting returns original message.
    $this->assertEquals(static::PLAIN_TEXT_MESSAGE, $profile->getEncryptionMethod()->decrypt($text, $profile->getEncryptionKey()->getKeyValue()));

  }

}
